@@ -150,20 +150,40 @@ cherokee_buffer_add_buffer (cherokee_buffer_t *buf, cherokee_buffer_t *buf2)
 }
 
 
+ret_t 
+cherokee_buffer_add_va_list (cherokee_buffer_t *buf, char *format, va_list args)
+{
+	cuint_t len;
+	cuint_t estimated_length;
+
+	estimated_length = cherokee_estimate_va_length (format, args);
+	cherokee_buffer_ensure_size (buf, buf->len + estimated_length + 2);
+
+	len = vsnprintf (buf->buf + buf->len, buf->size - buf->len -1, format, args);
+	
+#if 1
+	if (estimated_length < len)
+		PRINT_ERROR ("  -> '%s' -> '%s', esti=%d real=%d\n", format, buf->buf, estimated_length, len);
+#endif
+
+	if (len < 0) return ret_error;
+
+	buf->len += len;
+	return ret_ok;
+}
+
+
 ret_t 
 cherokee_buffer_add_va (cherokee_buffer_t *buf, char *format, ...)
 {
-	int     len;
+	ret_t   ret;
 	va_list ap;
-	CHEROKEE_TEMP (tmp, BUFFER_VA_LEN);
 
 	va_start (ap, format);
-	len = vsnprintf (tmp, tmp_size, format, ap);
+	ret = cherokee_buffer_add_va_list (buf, format, ap);
 	va_end (ap);
 
-	if (len >= BUFFER_VA_LEN-1) return ret_error;
-
-	return cherokee_buffer_add (buf, tmp, len);
+	return ret;
 }
 
 
@@ -240,8 +260,11 @@ cherokee_buffer_move_to_begin (cherokee_buffer_t *buf, int pos)
 {
 	int offset;
 
-	return_if_fail (pos <= buf->size, ret_error);
-	if (pos <= 0) return ret_ok;
+	if (pos <= 0) 
+		return ret_ok;
+
+	if (pos >= buf->len) 
+		return cherokee_buffer_clean(buf);
 
 	offset = MIN(pos, buf->len);
 	memmove (buf->buf, buf->buf+offset, (buf->len - offset) +1);
@@ -261,7 +284,7 @@ cherokee_buffer_move_to_begin (cherokee_buffer_t *buf, int pos)
 ret_t
 cherokee_buffer_ensure_size (cherokee_buffer_t *buf, int size)
 {
-	/* Is this a rew buffer?
+	/* Is this a new buffer?
 	 * In this case, just take the memory
 	 */
 	if (buf->buf == NULL) {
