@@ -31,6 +31,7 @@
 
 #include <cherokee/common.h>
 #include <stddef.h>
+#include <stdarg.h>
 
 
 CHEROKEE_BEGIN_DECLS
@@ -59,9 +60,10 @@ ret_t cherokee_buffer_ref_buffer         (cherokee_buffer_t  *buf, cherokee_buff
 
 ret_t cherokee_buffer_add                (cherokee_buffer_t  *buf, char *txt, size_t size);
 ret_t cherokee_buffer_add_va             (cherokee_buffer_t  *buf, char *format, ...);
+ret_t cherokee_buffer_add_va_list        (cherokee_buffer_t  *buf, char *format, va_list args);
 ret_t cherokee_buffer_add_char_n         (cherokee_buffer_t  *buf, char c, int n);
-ret_t cherokee_buffer_prepend            (cherokee_buffer_t  *buf, char *txt, size_t size);
 ret_t cherokee_buffer_add_buffer         (cherokee_buffer_t  *buf, cherokee_buffer_t *buf2);
+ret_t cherokee_buffer_prepend            (cherokee_buffer_t  *buf, char *txt, size_t size);
 
 ret_t cherokee_buffer_read_file          (cherokee_buffer_t  *buf, char *filename);
 ret_t cherokee_buffer_read_from_fd       (cherokee_buffer_t  *buf, int fd, size_t size, size_t *ret_size);
