@@ -57,6 +57,12 @@
  */
 #include <cherokee/downloader.h>
 
+/* Config library
+ */
+#include <cherokee/admin_client.h>
+#include <cherokee/config_value.h>
+
+
 #undef CHEROKEE_INSIDE_CHEROKEE_H
 
 #endif /* CHEROKEE_H */
