@@ -1005,7 +1005,9 @@ get_authorization (cherokee_connection_t *cnt,
 		   char                  *ptr,
 	           int                    ptr_len)
 {
-	cherokee_buffer_t *auth = NULL;
+	cherokee_buffer_t auth;
+
+	cherokee_buffer_init (&auth);
 
 	if (strncasecmp(ptr, "Basic ", 6) == 0) {
 		char *end;
@@ -1025,17 +1027,16 @@ get_authorization (cherokee_connection_t *cnt,
 			goto error;
 		}
 
-		cherokee_buffer_new (&auth);
-		cherokee_buffer_add (auth, ptr, end-ptr+1);
-		cherokee_buffer_decode_base64 (auth);
+		cherokee_buffer_add (&auth, ptr, end-ptr+1);
+		cherokee_buffer_decode_base64 (&auth);
 
-		end = strchr (auth->buf, ':');
+		end = strchr (auth.buf, ':');
 		if (end == NULL) {
 			goto error;
 		}
 		
-		cherokee_buffer_add (cnt->user, auth->buf, end - auth->buf);
-		cherokee_buffer_add (cnt->passwd, end+1, auth->len  - (end - auth->buf));		
+		cherokee_buffer_add (cnt->user, auth.buf, end - auth.buf);
+		cherokee_buffer_add (cnt->passwd, end+1, auth.len  - (end - auth.buf));		
 
 	} else if (strncasecmp(ptr, "Digest ", 7) == 0) {
 
@@ -1054,13 +1055,11 @@ get_authorization (cherokee_connection_t *cnt,
 		return ret_error;
 	}
 	
+	cherokee_buffer_mrproper (&auth);
 	return ret_ok;
 
 error:
-	if (auth != NULL) {
-		cherokee_buffer_free (auth);
-	}
-
+	cherokee_buffer_mrproper (&auth);
 	return ret_error;
 }
 
@@ -1719,7 +1718,13 @@ cherokee_connection_log_or_delay (cherokee_connection_t *conn)
 
 	if (conn->log_at_end == 0) {
 		if (conn->logger_ref != NULL) {
-			ret = cherokee_logger_write_access (conn->logger_ref, conn);
+			if (http_type_400(conn->error_code) ||
+			    http_type_500(conn->error_code)) 
+			{
+				ret = cherokee_logger_write_error (conn->logger_ref, conn);				
+			} else {
+				ret = cherokee_logger_write_access (conn->logger_ref, conn);
+			}
 			conn->log_at_end = 0;
 		}
 	}
