@@ -98,8 +98,11 @@ cherokee_downloader_init (cherokee_downloader_t *n)
 
 	/* Init the properties
 	 */
-	n->fdpoll = NULL;
-	n->phase  = downloader_phase_init;
+	n->fdpoll   = NULL;
+	n->phase    = downloader_phase_init;
+
+	n->post_ref  = NULL;
+	n->post_sent = 0;
 
 	/* Lengths
 	 */
@@ -161,27 +164,34 @@ cherokee_downloader_set_fdpoll (cherokee_downloader_t *downloader, cherokee_fdpo
 ret_t 
 cherokee_downloader_set_url (cherokee_downloader_t *downloader, cherokee_buffer_t *url_string)
 {
-	ret_t           ret;
-	cherokee_url_t *url;
+	ret_t                      ret;
+	cherokee_request_header_t *req;
 
-	url = downloader->request->url;
+	req = downloader->request;
 
 	/* Parse the string in a URL object
 	 */
-	ret = cherokee_url_parse (url, url_string);
+	ret = cherokee_url_parse (req->url, url_string);
 	if (unlikely(ret < ret_ok)) return ret;
 
+	/* Maybe add the post info
+	 */
+	if (downloader->post_ref != NULL) {
+		REQUEST_METHOD(req) = http_post;
+		REQUEST_POST(req)   = downloader->post_ref->len;
+	}
+	
 	/* Build the request header
 	 */
-	ret = cherokee_request_header_build_string (downloader->request, &downloader->request_header);
+	ret = cherokee_request_header_build_string (req, &downloader->request_header);
 	if (unlikely(ret < ret_ok)) return ret;
 
 	return ret_ok;
 }
 
 
-static ret_t
-downloader_init (cherokee_downloader_t *downloader)
+ret_t 
+cherokee_downloader_prepare (cherokee_downloader_t *downloader)
 {
 	int                 r;
 	ret_t               ret;
@@ -243,12 +253,18 @@ downloader_init (cherokee_downloader_t *downloader)
 		ret = (* downloader->callback.init) (downloader, downloader->callback.param[downloader_event_init]);
 	}
 
+	/* Update the phase
+	 */
+	if (downloader->phase == downloader_phase_init) {
+		downloader->phase = downloader_phase_send_headers;		
+	}
+
 	return ret_ok;
 }
 
 
 static ret_t
-downloader_header_send (cherokee_downloader_t *downloader)
+downloader_send_buffer (cherokee_downloader_t *downloader, cherokee_buffer_t *buf)
 {
 	ret_t              ret;
 	size_t             writed;
@@ -256,13 +272,13 @@ downloader_header_send (cherokee_downloader_t *downloader)
 
 	sock = downloader->socket;
 	
-	ret = cherokee_socket_write (sock, &downloader->request_header, &writed);
+	ret = cherokee_socket_write (sock, buf, &writed);
 	switch (ret) {
 	case ret_ok:
 		/* Drop the header that has been sent
 		 */
-		cherokee_buffer_drop_endding (&downloader->request_header, writed);
-		if (cherokee_buffer_is_empty (&downloader->request_header)) {
+		cherokee_buffer_drop_endding (buf, writed);
+		if (cherokee_buffer_is_empty (buf)) {
 			return ret_ok;
 		}
 
@@ -440,7 +456,7 @@ cherokee_downloader_step (cherokee_downloader_t *downloader)
 
 	switch (downloader->phase) {
 	case downloader_phase_init:
-		ret = downloader_init (downloader);
+		ret = cherokee_downloader_prepare (downloader);
 		if (unlikely(ret < ret_ok)) return ret;
 		
 		/* Everything is ok, go ahead!
@@ -456,16 +472,31 @@ cherokee_downloader_step (cherokee_downloader_t *downloader)
 			return ret_eagain;
 		}
 
-		ret = downloader_header_send (downloader);
+		ret = downloader_send_buffer (downloader, &downloader->request_header);
 		if (unlikely(ret != ret_ok)) return ret;
-		
+
+		/* Come on..
+		 */
+		downloader->phase = downloader_phase_send_post;
+
+	case downloader_phase_send_post:
+		if (downloader->post_ref != NULL) {
+			/* Can it write in the socket?
+			 */
+			if (! cherokee_fdpoll_check (downloader->fdpoll, SOCKET_FD(downloader->socket), 1)) {
+				return ret_eagain;
+			}
+
+			ret = downloader_send_buffer (downloader, downloader->post_ref);
+			if (unlikely(ret != ret_ok)) return ret;
+		}
+
 		/* Switch the socket to read-only mode..
 		 */
 		cherokee_fdpoll_set_mode (downloader->fdpoll, SOCKET_FD(downloader->socket), 0);
 
-		/* Come on..
-		 */
 		downloader->phase = downloader_phase_read_headers;
+		break;
 
 	case downloader_phase_read_headers:
 		/* Can it read in the socket?
@@ -564,3 +595,16 @@ cherokee_downloader_connect (cherokee_downloader_t *downloader, cherokee_downloa
 	
 	return ret_ok;
 }
+
+
+ret_t 
+cherokee_downloader_set_post (cherokee_downloader_t *downloader, cherokee_buffer_t *post)
+{
+	if (downloader->post_ref != NULL) {
+		PRINT_ERROR_S ("WARNING: Overwriting post info\n");
+	}
+
+	downloader->post_ref = post;
+	return ret_ok;
+}
+
