@@ -24,6 +24,7 @@
 
 #include "common-internal.h"
 #include "fcgi_manager.h"
+#include "fastcgi.h"
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -37,7 +38,7 @@ cherokee_fcgi_manager_new  (cherokee_fcgi_manager_t **fcgim)
 {
 	int   i;
 	ret_t ret;
-	CHEROKEE_NEW(n,fcgi_manager);
+	CHEROKEE_NEW_STRUCT (n,fcgi_manager);
 
 	/* Init
 	 */
@@ -45,9 +46,10 @@ cherokee_fcgi_manager_new  (cherokee_fcgi_manager_t **fcgim)
 	if (unlikely(ret != ret_ok)) return ret;
 
 	n->port = DEFAULT_PORT;
-
 	cherokee_buffer_init (&n->hostname);
-	cherokee_buffer_init (&n->launch_command);
+	cherokee_buffer_init (&n->read_buffer);
+
+	cherokee_buffer_ensure_size (&n->read_buffer, DEFAULT_READ_SIZE);
 
 	n->conn_poll_size = 0;
 	n->conn_poll      = (cherokee_connection_t **) malloc (
@@ -68,7 +70,7 @@ ret_t
 cherokee_fcgi_manager_free (cherokee_fcgi_manager_t *fcgim)
 {
 	cherokee_buffer_mrproper (&fcgim->hostname);
-	cherokee_buffer_mrproper (&fcgim->launch_command);
+	cherokee_buffer_mrproper (&fcgim->read_buffer);
 
 	if (fcgim->socket != NULL) {
 		cherokee_socket_free (fcgim->socket);
@@ -85,43 +87,6 @@ cherokee_fcgi_manager_free (cherokee_fcgi_manager_t *fcgim)
 }
 
 
-static ret_t
-spawn_server (cherokee_fcgi_manager_t *fcgim)
-{
-	int   re;
-	int   child;
-	char *argv[] = {"sh", "-c", NULL, NULL};
-	char *envp[] = {NULL};
-
-	if (cherokee_buffer_is_empty (&fcgim->launch_command))
-		return ret_not_found;
-
-	printf ("FastCGI server not running, launching: %s\n", fcgim->launch_command.buf);
-
-	child = fork();
-	switch (child) {
-	case 0:
-		cherokee_buffer_prepend (&fcgim->launch_command, "exec ", 5);
-		argv[2] = (char *)&(fcgim->launch_command.buf);
-
-		re = execve ("/bin/sh", argv, envp);
-		if (re < 0) {
-			PRINT_ERROR ("ERROR: Could spawn %s\n", fcgim->launch_command.buf);
-		}
-
-	case -1:
-		return ret_error;
-		
-	default:
-		sleep (1);
-		break;
-		
-	}
-
-	return ret_ok;
-}
-
-
 static ret_t
 connect_to_fastcgi_server (cherokee_fcgi_manager_t *fcgim)
 {
@@ -135,38 +100,25 @@ connect_to_fastcgi_server (cherokee_fcgi_manager_t *fcgim)
 
 	SOCKET_SIN_PORT(fcgim->socket) = htons(fcgim->port);
 
-	ret = cherokee_socket_connect (fcgim->socket);
-	if (ret != ret_ok) {
-		/* The server seems to be dows, spawn in
-		 */
-		ret = spawn_server (fcgim);
-		if (ret != ret_ok) return ret;
-
-		/* Reconnect
-		 */
-		ret = cherokee_socket_connect (fcgim->socket);
-		if (ret != ret_ok) return ret;
-	}	
-
-	return ret_ok;
+	return cherokee_socket_connect (fcgim->socket);
 }
 
 
 ret_t 
-cherokee_fcgi_manager_init (cherokee_fcgi_manager_t *fcgim, cherokee_buffer_t *host)
+cherokee_fcgi_manager_connect_to_srv (cherokee_fcgi_manager_t *fcgim, char *host)
 {
 	char *port;
 	ret_t ret;
 	
 	/* Parse host name
 	 */
-	port = strchr(host->buf, ':');
+	port = strchr(host, ':');
 	if (port == NULL) {
-		cherokee_buffer_add_buffer (&fcgim->hostname, host);
+		cherokee_buffer_add (&fcgim->hostname, host, strlen(host));
 	} else {
 		*port = '\0';
 		fcgim->port = atoi(port+1);
-		cherokee_buffer_add (&fcgim->hostname, host->buf, port - host->buf);
+		cherokee_buffer_add (&fcgim->hostname, host, port - host);
 		*port = ':';
 	}
 	
@@ -180,7 +132,49 @@ cherokee_fcgi_manager_init (cherokee_fcgi_manager_t *fcgim, cherokee_buffer_t *h
 
 
 ret_t 
-cherokee_fcgi_manager_register (cherokee_fcgi_manager_t *fcgim, cherokee_connection_t *conn, cuint_t *id)
+cherokee_fcgi_manager_spawn_srv (cherokee_fcgi_manager_t *fcgim, char *command)
+{
+	int                re;
+	int                child;
+	char              *argv[] = {"sh", "-c", NULL, NULL};
+	char              *envp[] = {NULL};
+	cherokee_buffer_t  tmp;
+
+	cherokee_buffer_init (&tmp);
+	cherokee_buffer_add_va (&tmp, "exec %s", command);
+
+	printf ("FastCGI server not running, launching: %s\n", command);
+
+	child = fork();
+	switch (child) {
+	case 0:
+		argv[2] = (char *)tmp.buf;
+
+		re = execve ("/bin/sh", argv, envp);
+		if (re < 0) {
+			PRINT_ERROR ("ERROR: Could spawn %s\n", tmp.buf);
+		}
+
+	case -1:
+		goto error;
+		
+	default:
+		sleep (1);
+		break;
+		
+	}
+
+	cherokee_buffer_mrproper (&tmp);
+	return ret_ok;
+
+error:
+	cherokee_buffer_mrproper (&tmp);
+	return ret_error;
+}
+
+
+ret_t 
+cherokee_fcgi_manager_register_conn (cherokee_fcgi_manager_t *fcgim, cherokee_connection_t *conn, cuint_t *id)
 {
 	cuint_t i;
 	cint_t  slot = -1;
@@ -220,13 +214,56 @@ cherokee_fcgi_manager_register (cherokee_fcgi_manager_t *fcgim, cherokee_connect
 ret_t 
 cherokee_fcgi_manager_send (cherokee_fcgi_manager_t *fcgim, cherokee_buffer_t *info, size_t *sent)
 {
-	return cherokee_socket_write (fcgim->socket, info, sent);
+	ret_t ret;
+
+	ret = cherokee_socket_write (fcgim->socket, info, sent);
+	if (ret != ret_ok) return ret;
+
+	cherokee_buffer_move_to_begin (info, *sent);
+	return ret_ok;
+}
+
+
+static ret_t
+process_read_buffer (cherokee_fcgi_manager_t *fcgim)
+{
+	cuint_t      offset;
+	FCGI_Header *header;
+	
+	offset = 0;
+	while (fcgim->read_buffer.len - offset >= sizeof(FCGI_EndRequestRecord))
+	{
+		cuint_t id;
+		cuint_t len;
+		
+		header = (FCGI_Header *)((&fcgim->read_buffer.buf) + offset);
+		id     = (header->requestIdB0 | (header->requestIdB1 << 8));
+		len    = (header->contentLengthB0 | (header->contentLengthB1 << 8)) + header->paddingLength;
+
+		printf ("leido id=%d len=%d\n", id, len);
+
+		offset += sizeof(FCGI_EndRequestRecord);
+	}
+
+	return ret_ok;
 }
 
 
 ret_t 
 cherokee_fcgi_manager_step (cherokee_fcgi_manager_t *fcgim)
 {
+	ret_t  ret;
+	size_t size;
+
+	/* Read from the FastCGI application
+	 */
+	ret = cherokee_socket_read (fcgim->socket, &fcgim->read_buffer, DEFAULT_READ_SIZE, &size);
+	if (ret != ret_ok) return ret;
+
+	/* Process the information
+	 */
+	ret = process_read_buffer (fcgim);
+
 	/* Read
 	 */
 	return ret_ok;
