@@ -26,6 +26,7 @@
 #include "handler_admin.h"
 #include "connection.h"
 #include "connection-protected.h"
+#include "config_value.h"
 
 
 cherokee_module_info_t MODULE_INFO(admin) = {
@@ -70,7 +71,9 @@ cherokee_handler_admin_free (cherokee_handler_admin_t *ahdl)
 ret_t 
 cherokee_handler_admin_init (cherokee_handler_admin_t *ahdl)
 {
-	cherokee_connection_t *conn = HANDLER_CONN(ahdl);
+	ret_t                    ret;
+	cherokee_config_value_t *conf;
+	cherokee_connection_t   *conn = HANDLER_CONN(ahdl);
 
 	/* Check for the post info
 	 */
@@ -81,6 +84,15 @@ cherokee_handler_admin_init (cherokee_handler_admin_t *ahdl)
 
 	/* Parse the Post info
 	 */
+	ret = cherokee_config_value_parse (&conf, conn->post->buf);
+	if (ret != ret_ok) {
+		cherokee_config_value_free (conf);
+		return ret;
+	}
+
+	/* What is it asking?
+	 */
+	
 
 	return ret_ok;
 }
