@@ -34,9 +34,11 @@
 
 #include "buffer.h"
 #include "handler.h"
+#include "cgi.h"
 
 #define ENV_VAR_NUM 30
 
+
 typedef struct {
 	cherokee_handler_t handler;
 
@@ -84,9 +86,6 @@ ret_t cherokee_handler_cgi_split_pathinfo (cherokee_handler_cgi_t *cgi,
 
 void  cherokee_handler_cgi_add_parameter  (cherokee_handler_cgi_t *cgi, char *name);
 
-void  cherokee_handler_cgi_add_env        (cherokee_handler_cgi_t *cgi,
-				 	  char *string, int string_len);
-
 void  cherokee_handler_cgi_add_env_pair   (cherokee_handler_cgi_t *cgi, 
 				          char *name,    int name_len,
 					  char *content, int content_len);
