@@ -138,7 +138,6 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 			/* Check if the index is fullpath
 			 */
 			if (*index == '/') {
-				// cherokee_buffer_t *new_local_dir;
 				cherokee_buffer_t new_local_dir;
 
 				/* This means there is a configuration entry like:
@@ -184,7 +183,7 @@ cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table
 			
 			/* Add the index file to the request and clean up
 			 */
-			cherokee_buffer_clean (conn->local_directory);
+			cherokee_buffer_drop_endding (conn->local_directory,  conn->request->len);
 			cherokee_buffer_add (conn->request, index, index_len);
 
 			return ret_eagain;
