@@ -53,39 +53,45 @@ cherokee_module_info_t MODULE_INFO(dirlist) = {
 
 struct file_entry {
 	struct list_head list_entry;
-	struct dirent    info;
 	struct stat      stat;
+	cuint_t          name_len;
+	struct dirent    info;          /* It *must* be the last entry */
 };
 typedef struct file_entry file_entry_t;
 
 
-
 file_entry_t *
 new_file_entry (cherokee_buffer_t *path, struct dirent *file, cherokee_handler_dirlist_t *dhdl)
 {
 	int           re;
 	file_entry_t *n;
 	char         *name;
-	int           name_len;
+	cuint_t       extra;
+
+#ifdef PATH_MAX
+	extra = PATH_MAX +1;
+#else
+	extra = pathconf("/", _PC_PATH_MAX) +1;
+#endif
 
 	/* Get a new object
 	 */
-	n = malloc (sizeof(file_entry_t));
+	n = malloc (sizeof(file_entry_t) + extra);
 	if (unlikely(n == NULL)) return NULL;
 
 	INIT_LIST_HEAD(&n->list_entry);
-	memcpy (&n->info, file, sizeof(struct dirent));
+	memcpy (&n->info, file, sizeof(struct dirent) + extra);
 
 	/* Build the local path, stat and clean
 	 */
 	name = (char *)&file->d_name;
-	name_len = strlen(name);
-	cherokee_buffer_add (path, name, name_len);
+	n->name_len = strlen(name);
+	cherokee_buffer_add (path, name, n->name_len);
 
 	/* Check the filename size
 	 */
-	if (name_len > dhdl->longest_filename)
-		dhdl->longest_filename = name_len;
+	if (n->name_len > dhdl->longest_filename)
+		dhdl->longest_filename = n->name_len;
 
 	/* Path
 	 */
@@ -97,7 +103,7 @@ new_file_entry (cherokee_buffer_t *path, struct dirent *file, cherokee_handler_d
 
 	/* Clean up and exit
 	 */
-	cherokee_buffer_drop_endding (path, name_len);
+	cherokee_buffer_drop_endding (path, n->name_len);
 	return n;
 }
 
@@ -557,11 +563,9 @@ render_file (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer, file_e
 {
 	int               is_dir;
 	char             *icon;
-	cherokee_icons_t *icons = HANDLER_SRV(dhdl)->icons;
-	char             *name  = (char *) &file->info.d_name;
-	int               name_len;
-
-	name_len = strlen (name);
+	cherokee_icons_t *icons    = HANDLER_SRV(dhdl)->icons;
+	char             *name     = (char *) &file->info.d_name;
+	cuint_t           name_len = file->name_len;
 
 	/* Ignore backup files
 	 */
@@ -659,6 +663,7 @@ render_file (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer, file_e
 ret_t
 cherokee_handler_dirlist_step (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer)
 {
+	cuint_t                port;
 	cherokee_connection_t *conn = HANDLER_CONN(dhdl);
 
 	/* Page header
@@ -704,10 +709,15 @@ cherokee_handler_dirlist_step (cherokee_handler_dirlist_t *dhdl, cherokee_buffer
 	 */
 	cherokee_buffer_add (buffer, "</pre><hr>\n", 11);
 
+	if (HANDLER_CONN(dhdl)->socket->is_tls == non_TLS)
+ 		port = HANDLER_SRV(dhdl)->port;
+	else 
+ 		port = HANDLER_SRV(dhdl)->port_tls;
+
 	if (CONN_SRV(conn)->server_token <= cherokee_version_product) {
-		cherokee_buffer_add_version (buffer, HANDLER_SRV(dhdl)->port, ver_full_html);
+		cherokee_buffer_add_version (buffer, port, ver_full_html);
 	} else {
-		cherokee_buffer_add_version (buffer, HANDLER_SRV(dhdl)->port, ver_port_html);
+		cherokee_buffer_add_version (buffer, port, ver_port_html);
 	}
 
 	cherokee_buffer_add (buffer, "\n</body></html>", 15);
