@@ -70,12 +70,23 @@ cherokee_handler_fastcgi_new (cherokee_handler_t **hdl, cherokee_connection_t *c
 
 	/* Init
 	 */	
+	n->id              = 0;
+	n->manager_ref     = NULL;
+	n->host_ref        = NULL;
+	n->interpreter_ref = NULL;
+	
 	cherokee_buffer_init (&n->write_buffer);
 	cherokee_buffer_init (&n->incoming_buffer);
 	cherokee_buffer_init (&n->environment);
 
 	if (properties) {
-		;
+		char *val;
+
+		val = cherokee_table_get_val (properties, "server");
+		if (val) n->host_ref = val;
+
+		val = cherokee_table_get_val (properties, "interpreter");
+		if (val) n->interpreter_ref = val;
 	}
 
 	/* Return
@@ -98,8 +109,6 @@ cherokee_handler_fastcgi_free (cherokee_handler_fastcgi_t *hdl)
 
 
 
-
-
 static void
 add_env_pair (cherokee_buffer_t *buf, 
 	      char *key, int key_len,
@@ -169,9 +178,6 @@ build_initial_packages (cherokee_handler_fastcgi_t *fcgi)
 	ret = cherokee_cgi_build_basic_env (conn, (cherokee_cgi_set_env_pair_t) add_env_pair, &tmp, &fcgi->write_buffer);
 	if (unlikely (ret != ret_ok)) return ret;
 
-	/*
-	 */
-	
 	cherokee_buffer_mrproper (&tmp);
 
 	return ret_ok;
@@ -181,13 +187,61 @@ build_initial_packages (cherokee_handler_fastcgi_t *fcgi)
 ret_t 
 cherokee_handler_fastcgi_init (cherokee_handler_fastcgi_t *fcgi)
 {
-	ret_t ret;
+	ret_t                  ret;
+	size_t                 sent   = 0;
+	cherokee_connection_t *conn   = HANDLER_CONN(fcgi);
+	cherokee_thread_t     *thread = HANDLER_THREAD(fcgi);
+
+	if (fcgi->host_ref == NULL) {
+		PRINT_ERROR_S ("ERROR: FastCGI without Host\n");
+		return ret_error;
+	}
+
+	/* Look for the FCGI managers table
+	 */
+	ret = cherokee_table_get (thread->fcgi_managers, fcgi->host_ref, (void **)&fcgi->manager_ref);
+	if (ret == ret_not_found) {
+		cherokee_fcgi_manager_t *n;
+
+		if (fcgi->interpreter_ref == NULL) {
+			PRINT_ERROR_S ("Could connect to FastCGI server and hadn't interpreter to launch\n");
+			return ret_error;
+		}
+
+		/* Create a new manager object
+		 */
+		ret = cherokee_fcgi_manager_new (&n);
+		if (unlikely (ret != ret_ok)) return ret;
+		
+                /* Assign the object to that path
+		 */
+		cherokee_table_add (thread->fcgi_managers, fcgi->host_ref, n);
+		fcgi->manager_ref = n;
+
+		/* Launch a new FastCGI server and connect to it
+		 */
+		ret = cherokee_fcgi_manager_spawn_srv (n, fcgi->interpreter_ref);
+		if (unlikely (ret != ret_ok)) return ret;
+
+		ret = cherokee_fcgi_manager_connect_to_srv (n, fcgi->host_ref);
+		if (unlikely (ret != ret_ok)) return ret;
+	}
+
+	/* Register this connection in the FastCGI manager
+	 */
+	ret = cherokee_fcgi_manager_register_conn (fcgi->manager_ref, conn, &fcgi->id);
+	if (unlikely (ret != ret_ok)) return ret;
 
 	/* Send the first packet
 	 */
 	ret = build_initial_packages (fcgi);
 	if (unlikely (ret != ret_ok)) return ret;
 	
+	ret = cherokee_fcgi_manager_send (fcgi->manager_ref, &fcgi->write_buffer, &sent);
+	if (unlikely (ret != ret_ok)) return ret;
+
+	cherokee_buffer_move_to_begin (&fcgi->write_buffer, sent);
+
 	return ret_ok;
 }
 
@@ -195,27 +249,38 @@ cherokee_handler_fastcgi_init (cherokee_handler_fastcgi_t *fcgi)
 ret_t 
 cherokee_handler_fastcgi_step (cherokee_handler_fastcgi_t *fcgi, cherokee_buffer_t *buffer)
 {
-/* 	ret_t  ret; */
-/* 	size_t done; */
+	ret_t ret;
+	size_t done;
+
+	return_if_fail (fcgi->manager_ref != NULL, ret_error);
 	
-/* 	/\* It has something to send */
-/* 	 *\/ */
-/* 	if (! cherokee_buffer_is_empty (&fcgi->write_buffer)) { */
-/* 		ret = cherokee_socket_write (fcgi->socket, &fcgi->write_buffer, &done); */
-/* 		switch (ret) { */
-/* 		case ret_ok: */
-/* 			break; */
-/* 		case ret_eagain: */
-/* 			return ret_eagain; */
-/* 		case ret_eof: */
-/* 		case ret_error: */
-/* 			return ret_error; */
-/* 		default: */
-/* 			SHOULDNT_HAPPEN; */
-/* 		} */
+	/* It has something to send
+	 */
+	if (! cherokee_buffer_is_empty (&fcgi->write_buffer)) {
+		ret = cherokee_fcgi_manager_send (fcgi->manager_ref, &fcgi->write_buffer, &done);
+		switch (ret) {
+		case ret_ok:
+			if (cherokee_buffer_is_empty (&fcgi->write_buffer))
+				return ret_ok;
+			else 
+				return ret_eagain;
+
+		case ret_eagain:
+			return ret_eagain;
+
+		case ret_eof:
+		case ret_error:
+			return ret_error;
+		default:
+			SHOULDNT_HAPPEN;
+		}
+	}
+
+	/* Lets read from the FastCGI server
+	 * As side effect it could update more connections in this call
+	 */
+	ret = cherokee_fcgi_manager_step (fcgi->manager_ref);
 
-/* 		cherokee_buffer_move_to_begin (&fcgi->write_buffer, done); */
-/* 	} */
 
 /* 	/\* Lets read */
 /* 	 *\/ */
