@@ -171,7 +171,7 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 
 	/* Open the file and stat
 	 */
-	n->fd = open (conn->local_directory->buf, O_RDONLY);
+	n->fd = open (conn->local_directory->buf, CHE_O_READ);
 
 	if (n->fd != -1) {
 		ret = fstat (n->fd, &n->info);
