@@ -111,7 +111,7 @@ get_nearest (cherokee_buffer_t *local,
 
 	/* Build new web request
 	 */
-	cherokee_buffer_make_empty (req);
+	cherokee_buffer_clean (req);
 	cherokee_buffer_add (req, request->buf, rest - request->buf);
 	cherokee_buffer_add (req, tmp, strlen(tmp));
 
