@@ -65,7 +65,7 @@ static char *month[]   = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 #define IN_ADDR(c) ((struct in_addr) (c).sin_addr)
 
 
-cherokee_module_info_t cherokee_w3c_info = {
+cherokee_module_info_t MODULE_INFO(w3c) = {
 	cherokee_logger,            /* type     */
 	cherokee_logger_w3c_new     /* new func */
 };
@@ -290,20 +290,18 @@ cherokee_logger_w3c_write_access (cherokee_logger_w3c_t *logger, cherokee_connec
 
 
 
-/*   Library init function
+/* Library init function
  */
 
-static int _w3c_is_init = 0;
+static cherokee_boolean_t _w3c_is_init = false;
 
 void
-w3c_init ()
+MODULE_INIT(w3c) (cherokee_module_loader_t *loader)
 {
 	/* Init flag
 	 */
-	if (_w3c_is_init) {
-		return;
-	}
-	_w3c_is_init = 1;
+	if (_w3c_is_init) return;
+	_w3c_is_init = true;
 }
 
 
