@@ -44,7 +44,7 @@
 #include "server.h"
 #include "list.h"
 #include "fdpoll.h"
-#include "handler_table.h"
+#include "dirs_table.h"
 #include "virtual_server.h"
 #include "encoder_table.h"
 #include "logger_table.h"
