@@ -612,11 +612,11 @@ set_server_socket_opts (int socket)
 	 * attempts TCP will make to complete the connection. This option should 
 	 * not be used in code intended to be portable.
 	 *
-	 * Give clients 10s to send first data packet 
+	 * Give clients 20s to send first data packet 
 	 */
-#if defined (TCP_DEFER_ACCEPT)
-	on = 10;
-	re = setsockopt (socket, SOL_SOCKET, TCP_DEFER_ACCEPT, &on, sizeof(on));
+#if defined (TCP_DEFER_ACCEPT) && defined(SOL_TCP)
+	on = 20;
+	re = setsockopt (socket, SOL_TCP, TCP_DEFER_ACCEPT, &on, sizeof(on));
 	if (re != 0) return ret_error;
 #endif
 
@@ -719,8 +719,14 @@ print_banner (cherokee_server_t *srv)
 	/* First line
 	 */
 	cherokee_buffer_add_va (n, "Cherokee Web Server %s: ", PACKAGE_VERSION);
-	cherokee_buffer_add_va (n, "Listening on port %d", srv->port);
-		
+
+	if (srv->socket_tls != -1 && srv->socket != -1) {
+		cherokee_buffer_add_va (n, "Listening on ports %d and %d", srv->port, srv->port_tls);
+	} else {
+		cherokee_buffer_add_va (n, "Listening on port %d", 
+					(srv->socket != -1)? srv->port : srv->port_tls);
+	}
+
 	if (srv->chrooted) {
 		cherokee_buffer_add (n, ", chrooted", 10);
 	}
