@@ -1,20 +1,20 @@
 import random
 import string
 from base import *
+from util import *
+
+MAGIC = str_random(10*1024)
 
-MAGIC=""
-for i in xrange(100*1024):
-    MAGIC += random.choice(string.letters)
 
 class Test (TestBase):
     def __init__ (self):
         TestBase.__init__ (self)
-        self.name = "100k random"
+        self.name = "10k random"
 
-        self.request          = "GET /100k HTTP/1.0\r\n"
+        self.request          = "GET /10k HTTP/1.0\r\n"
         self.expected_error   = 200
         self.expected_content = MAGIC
 
     def Prepare (self, www):
-        self.WriteFile (www, "100k", 0444, MAGIC)
+        self.WriteFile (www, "10k", 0444, MAGIC)
 
