@@ -1,13 +1,12 @@
 import random
 import string
 from base import *
+from util import *
 
+MAGIC  = str_random (100*1024)
 LENGTH = 100*1024
 OFFSET = 15
 
-MAGIC = ""
-for i in xrange(LENGTH):
-    MAGIC += random.choice(string.letters)
 
 class Test (TestBase):
     def __init__ (self):
