@@ -1,14 +1,13 @@
 import random
 import string
 from base import *
+from util import *
 
+MAGIC   = str_random (100*1024)
 LENGTH  = 100*1024
 OFFSET1 = 15*1024
 OFFSET2 = 40*1024
 
-MAGIC = ""
-for i in xrange(LENGTH):
-    MAGIC += random.choice(string.letters)
 
 class Test (TestBase):
     def __init__ (self):
