@@ -1,10 +1,10 @@
 import random
 import string
 from base import *
+from util import *
+
+MAGIC = letters_random (100*1024)
 
-MAGIC=''
-for i in xrange(100*1024):
-    MAGIC += random.choice(string.letters)
 
 class Test (TestBase):
     def __init__ (self):
