@@ -1,3 +1,4 @@
+import os
 from base import *
 
 class Test (TestBase):
@@ -16,9 +17,6 @@ class Test (TestBase):
                                   "ScriptName /super_test_index.php",
                                   "RequestUri /directoryindex3/inside/"]
 
-    def Precondition (self):
-        return os.path.exists (PHPCGI_PATH)
-
     def JustBefore (self, www):
         self.WriteFile (www, "super_test_index.php", 0444, """<?php
                         echo "DocumentRoot ".$_SERVER[DOCUMENT_ROOT]."\n";
@@ -29,3 +27,13 @@ class Test (TestBase):
     def JustAfter (self, www):
         self.Remove (www, "super_test_index.php")
 
+    def Precondition (self):
+        if os.path.exists (PHPCGI_PATH) is False:
+            return False
+
+        f = os.popen("ps -p %d -L | wc -l" % (os.getpid()))
+        threads = int(f.read()[:-1])
+        f.close()
+
+        return (threads <= 2)
+        
